@@ -43,16 +43,31 @@ func (h *ListenerHandler) NewConnection(ctx context.Context, conn net.Conn, meta
 	return h.ListenerHandler.NewConnection(ctx, conn, metadata)
 }
 
+func (h *ListenerHandler) NewPacket(ctx context.Context, key netip.AddrPort, buffer *buf.Buffer, metadata M.Metadata, init func(natConn network.PacketConn) network.PacketWriter) {
+	if h.ShouldHijackDns(metadata.Destination.AddrPort()) {
+		log.Debugln("[DNS] hijack udp:%s from %s", metadata.Destination.String(), metadata.Source.String())
+		writer := init(nil)
+		rwOptions := network.ReadWaitOptions{
+			FrontHeadroom: network.CalculateFrontHeadroom(writer),
+			RearHeadroom:  network.CalculateRearHeadroom(writer),
+			MTU:           resolver.SafeDnsPacketSize,
+		}
+		go relayDnsPacket(ctx, buffer, rwOptions, metadata.Destination, nil, &writer)
+		return
+	}
+	h.ListenerHandler.NewPacket(ctx, key, buffer, metadata, init)
+}
+
 func (h *ListenerHandler) NewPacketConnection(ctx context.Context, conn network.PacketConn, metadata M.Metadata) error {
 	if h.ShouldHijackDns(metadata.Destination.AddrPort()) {
 		log.Debugln("[DNS] hijack udp:%s from %s", metadata.Destination.String(), metadata.Source.String())
 		defer func() { _ = conn.Close() }()
 		mutex := sync.Mutex{}
-		conn2 := conn // a new interface to set nil in defer
+		var writer network.PacketWriter = conn // a new interface to set nil in defer
 		defer func() {
 			mutex.Lock() // this goroutine must exit after all conn.WritePacket() is not running
 			defer mutex.Unlock()
-			conn2 = nil
+			writer = nil
 		}()
 		rwOptions := network.ReadWaitOptions{
 			FrontHeadroom: network.CalculateFrontHeadroom(conn),
@@ -89,43 +104,47 @@ func (h *ListenerHandler) NewPacketConnection(ctx context.Context, conn network.
 				}
 				return err
 			}
-			go func() {
-				ctx, cancel := context.WithTimeout(ctx, resolver.DefaultDnsRelayTimeout)
-				defer cancel()
-				inData := readBuff.Bytes()
-				writeBuff := readBuff
-				writeBuff.Resize(writeBuff.Start(), 0)
-				if len(writeBuff.FreeBytes()) < resolver.SafeDnsPacketSize { // only create a new buffer when space don't enough
-					writeBuff = rwOptions.NewPacketBuffer()
-				}
-				msg, err := resolver.RelayDnsPacket(ctx, inData, writeBuff.FreeBytes())
-				if writeBuff != readBuff {
-					readBuff.Release()
-				}
-				if err != nil {
-					writeBuff.Release()
-					return
-				}
-				writeBuff.Truncate(len(msg))
-				mutex.Lock()
-				defer mutex.Unlock()
-				conn := conn2
-				if conn == nil {
-					writeBuff.Release()
-					return
-				}
-				err = conn.WritePacket(writeBuff, dest) // WritePacket will release writeBuff
-				if err != nil {
-					writeBuff.Release()
-					return
-				}
-			}()
+			go relayDnsPacket(ctx, readBuff, rwOptions, dest, &mutex, &writer)
 		}
 		return nil
 	}
 	return h.ListenerHandler.NewPacketConnection(ctx, conn, metadata)
 }
 
+func relayDnsPacket(ctx context.Context, readBuff *buf.Buffer, rwOptions network.ReadWaitOptions, dest M.Socksaddr, mutex *sync.Mutex, writer *network.PacketWriter) {
+	ctx, cancel := context.WithTimeout(ctx, resolver.DefaultDnsRelayTimeout)
+	defer cancel()
+	inData := readBuff.Bytes()
+	writeBuff := readBuff
+	writeBuff.Resize(writeBuff.Start(), 0)
+	if len(writeBuff.FreeBytes()) < resolver.SafeDnsPacketSize { // only create a new buffer when space don't enough
+		writeBuff = rwOptions.NewPacketBuffer()
+	}
+	msg, err := resolver.RelayDnsPacket(ctx, inData, writeBuff.FreeBytes())
+	if writeBuff != readBuff {
+		readBuff.Release()
+	}
+	if err != nil {
+		writeBuff.Release()
+		return
+	}
+	writeBuff.Truncate(len(msg))
+	if mutex != nil {
+		mutex.Lock()
+		defer mutex.Unlock()
+	}
+	conn := *writer
+	if conn == nil {
+		writeBuff.Release()
+		return
+	}
+	err = conn.WritePacket(writeBuff, dest) // WritePacket will release writeBuff
+	if err != nil {
+		writeBuff.Release()
+		return
+	}
+}
+
 func (h *ListenerHandler) TypeMutation(typ C.Type) *ListenerHandler {
 	handle := *h
 	handle.ListenerHandler = h.ListenerHandler.TypeMutation(typ)
