@@ -6,7 +6,7 @@ import (
 
 	"github.com/metacubex/mihomo/component/iface"
 
-	"github.com/metacubex/sing-tun/control"
+	"github.com/metacubex/sing/common/control"
 )
 
 type defaultInterfaceFinder struct{}
