@@ -69,7 +69,7 @@ func (s *Server) Close() error {
 
 type serverHandler struct {
 	*Server
-	quicConn quic.EarlyConnection
+	quicConn *quic.Conn
 	uuid     uuid.UUID
 
 	v4Handler common.ServerHandler
@@ -138,7 +138,7 @@ func (s *serverHandler) handleMessage() (err error) {
 
 func (s *serverHandler) handleStream() (err error) {
 	for {
-		var quicStream quic.Stream
+		var quicStream *quic.Stream
 		quicStream, err = s.quicConn.AcceptStream(context.Background())
 		if err != nil {
 			return err
@@ -175,7 +175,7 @@ func (s *serverHandler) handleStream() (err error) {
 
 func (s *serverHandler) handleUniStream() (err error) {
 	for {
-		var stream quic.ReceiveStream
+		var stream *quic.ReceiveStream
 		stream, err = s.quicConn.AcceptUniStream(context.Background())
 		if err != nil {
 			return err
