@@ -14,6 +14,7 @@ import (
 	atomic2 "github.com/metacubex/mihomo/common/atomic"
 	N "github.com/metacubex/mihomo/common/net"
 	"github.com/metacubex/mihomo/common/pool"
+	"github.com/metacubex/mihomo/common/xsync"
 	tlsC "github.com/metacubex/mihomo/component/tls"
 	C "github.com/metacubex/mihomo/constant"
 	"github.com/metacubex/mihomo/log"
@@ -21,7 +22,6 @@ import (
 
 	"github.com/metacubex/quic-go"
 	"github.com/metacubex/randv2"
-	"github.com/puzpuzpuz/xsync/v3"
 )
 
 type ClientOption struct {
@@ -42,13 +42,13 @@ type clientImpl struct {
 	*ClientOption
 	udp bool
 
-	quicConn  quic.Connection
+	quicConn  *quic.Conn
 	connMutex sync.Mutex
 
 	openStreams atomic.Int64
 	closed      atomic.Bool
 
-	udpInputMap *xsync.MapOf[uint32, net.Conn]
+	udpInputMap xsync.Map[uint32, net.Conn]
 
 	// only ready for PoolClient
 	dialerRef   C.Dialer
@@ -71,7 +71,7 @@ func (t *clientImpl) SetLastVisited(last time.Time) {
 	t.lastVisited.Store(last)
 }
 
-func (t *clientImpl) getQuicConn(ctx context.Context, dialer C.Dialer, dialFn common.DialFunc) (quic.Connection, error) {
+func (t *clientImpl) getQuicConn(ctx context.Context, dialer C.Dialer, dialFn common.DialFunc) (*quic.Conn, error) {
 	t.connMutex.Lock()
 	defer t.connMutex.Unlock()
 	if t.quicConn != nil {
@@ -81,7 +81,7 @@ func (t *clientImpl) getQuicConn(ctx context.Context, dialer C.Dialer, dialFn co
 	if err != nil {
 		return nil, err
 	}
-	var quicConn quic.Connection
+	var quicConn *quic.Conn
 	if t.ReduceRtt {
 		quicConn, err = transport.DialEarly(ctx, addr, t.TlsConfig, t.QuicConfig)
 	} else {
@@ -113,7 +113,7 @@ func (t *clientImpl) getQuicConn(ctx context.Context, dialer C.Dialer, dialFn co
 	return quicConn, nil
 }
 
-func (t *clientImpl) sendAuthentication(quicConn quic.Connection) (err error) {
+func (t *clientImpl) sendAuthentication(quicConn *quic.Conn) (err error) {
 	defer func() {
 		t.deferQuicConn(quicConn, err)
 	}()
@@ -138,12 +138,12 @@ func (t *clientImpl) sendAuthentication(quicConn quic.Connection) (err error) {
 	return nil
 }
 
-func (t *clientImpl) handleUniStream(quicConn quic.Connection) (err error) {
+func (t *clientImpl) handleUniStream(quicConn *quic.Conn) (err error) {
 	defer func() {
 		t.deferQuicConn(quicConn, err)
 	}()
 	for {
-		var stream quic.ReceiveStream
+		var stream *quic.ReceiveStream
 		stream, err = quicConn.AcceptUniStream(context.Background())
 		if err != nil {
 			return err
@@ -189,7 +189,7 @@ func (t *clientImpl) handleUniStream(quicConn quic.Connection) (err error) {
 	}
 }
 
-func (t *clientImpl) handleMessage(quicConn quic.Connection) (err error) {
+func (t *clientImpl) handleMessage(quicConn *quic.Conn) (err error) {
 	defer func() {
 		t.deferQuicConn(quicConn, err)
 	}()
@@ -237,14 +237,14 @@ func (t *clientImpl) handleMessage(quicConn quic.Connection) (err error) {
 	}
 }
 
-func (t *clientImpl) deferQuicConn(quicConn quic.Connection, err error) {
+func (t *clientImpl) deferQuicConn(quicConn *quic.Conn, err error) {
 	var netError net.Error
 	if err != nil && errors.As(err, &netError) {
 		t.forceClose(quicConn, err)
 	}
 }
 
-func (t *clientImpl) forceClose(quicConn quic.Connection, err error) {
+func (t *clientImpl) forceClose(quicConn *quic.Conn, err error) {
 	t.connMutex.Lock()
 	defer t.connMutex.Unlock()
 	if quicConn == nil {
@@ -422,7 +422,6 @@ func NewClient(clientOption *ClientOption, udp bool, dialerRef C.Dialer) *Client
 		ClientOption: clientOption,
 		udp:          udp,
 		dialerRef:    dialerRef,
-		udpInputMap:  xsync.NewMapOf[uint32, net.Conn](),
 	}
 	c := &Client{ci}
 	runtime.SetFinalizer(c, closeClient)
